/*
 * @(#) OpenFileAction.java Oct 23, 2003 Copyright (c) 2002-2005 Delft
 * University of Technology Jaffalaan 5, 2628 BX Delft, the Netherlands. All
 * rights reserved. This software is proprietary information of Delft University
 * of Technology The code is published under the Lesser General Public License
 */
package nl.tudelft.simulation.dsol.gui.menu.actions;

import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.rmi.RemoteException;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.KeyStroke;

import nl.tudelft.simulation.dsol.gui.DSOLApplicationInterface;
import nl.tudelft.simulation.dsol.gui.windows.FileChooserFrame;
import nl.tudelft.simulation.event.Event;
import nl.tudelft.simulation.event.EventInterface;
import nl.tudelft.simulation.event.EventListenerInterface;
import nl.tudelft.simulation.logger.Logger;

/**
 * The OpenFile action <br>
 * (c) copyright 2002-2005 <a href="http://www.simulation.tudelft.nl">Delft
 * University of Technology </a>, the Netherlands. <br>
 * See for project information <a
 * href="http://www.simulation.tudelft.nl">www.simulation.tudelft.nl </a> <br>
 * License of use: <a href="http://www.gnu.org/copyleft/lesser.html">Lesser
 * General Public License (LGPL) </a>, no warranty.
 * 
 * @version $Revision: 1.8 $ $Date: 2005/08/09 10:41:04 $
 * @author <a href="http://www.peter-jacobs.com">Peter Jacobs </a>
 */

public class OpenFileAction extends AbstractAction implements
        EventListenerInterface
{
    /** the parent */
    private DSOLApplicationInterface application = null;

    /**
     * constructs a new OpenFileAction
     * 
     * @param application the parent application
     */
    public OpenFileAction(final DSOLApplicationInterface application)
    {
        super("Open File");
        this.application = application;
        this.putValue(Action.MNEMONIC_KEY, new Integer('O'));
        this.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke('O',
                InputEvent.CTRL_DOWN_MASK));
        this.putValue(Action.SMALL_ICON, new ImageIcon(this.getClass()
                .getResource("/toolbarButtonGraphics/general/Open16.gif")));
        this.setEnabled(false);
        try
        {
            this.application.addListener(this,
                    DSOLApplicationInterface.EXPERIMENT_CHANGED_EVENT, false);
            this.notify(new Event(
                    DSOLApplicationInterface.EXPERIMENT_CHANGED_EVENT,
                    this.application, this.application.getExperimentalFrame()));
        } catch (RemoteException exception)
        {
            Logger.warning(this, "OpenFileAction", exception);
        }
    }

    /**
     * @see java.awt.event.ActionListener
     *      #actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(final ActionEvent event)
    {
        new FileChooserFrame(this.application, "/");
    }

    /**
     * @see nl.tudelft.simulation.event.EventListenerInterface
     *      #notify(nl.tudelft.simulation.event.EventInterface)
     */
    public void notify(final EventInterface event)
    {
        if (event.getSource().equals(this.application)
                && event.getType().equals(
                        DSOLApplicationInterface.EXPERIMENT_CHANGED_EVENT))
        {
            if (event.getContent() == null)
            {
                this.setEnabled(true);
            } else
            {
                this.setEnabled(false);
            }
        }
    }

    /**
     * @see java.lang.Object#finalize()
     */
    @Override
    protected void finalize() throws Throwable
    {
        this.application.removeListener(this,
                DSOLApplicationInterface.EXPERIMENT_CHANGED_EVENT);
        super.finalize();
    }
}