/*
 * @(#) ExperimentTree.java Oct 14, 2003 Copyright (c) 2002-2005 Delft
 * University of Technology Jaffalaan 5, 2628 BX Delft, the Netherlands. All
 * rights reserved. This software is proprietary information of Delft University
 * of Technology The code is published under the Lesser General Public License
 */
package nl.tudelft.simulation.dsol.gui.panels;

import java.util.Calendar;
import java.util.Enumeration;
import java.util.Map;
import java.util.Properties;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

import nl.tudelft.simulation.dsol.experiment.Experiment;
import nl.tudelft.simulation.dsol.experiment.ExperimentalFrame;
import nl.tudelft.simulation.dsol.experiment.Replication;
import nl.tudelft.simulation.dsol.experiment.Treatment;
import nl.tudelft.simulation.jstats.streams.StreamInterface;

/**
 * ExperimentTree <br>
 * (c) copyright 2002-2005 <a href="http://www.simulation.tudelft.nl">Delft
 * University of Technology </a>, the Netherlands. <br>
 * See for project information <a
 * href="http://www.simulation.tudelft.nl">www.simulation.tudelft.nl </a> <br>
 * License of use: <a href="http://www.gnu.org/copyleft/lesser.html">Lesser
 * General Public License (LGPL) </a>, no warranty.
 * 
 * @version $Revision: 1.5 $ $Date: 2005/07/04 12:23:14 $
 * @author <a href="http://www.peter-jacobs.com">Peter Jacobs </a>
 */
public class ExperimentTree extends DefaultTreeModel
{
    /**
     * constructs a new ExperimentTree
     * 
     * @param experimentalFrame experimentalFrame
     */
    public ExperimentTree(final ExperimentalFrame experimentalFrame)
    {
        super(new DefaultMutableTreeNode());
        DefaultMutableTreeNode root = (DefaultMutableTreeNode) this.getRoot();
        root.add(new DefaultMutableTreeNode(experimentalFrame.getUrl()));
        for (Experiment experiment : experimentalFrame.getExperiments())
        {
            root.add(this.getExperiment(experiment));
        }
    }

    /**
     * converts the replication in a tree node
     * 
     * @param replication the replication
     * @return DefaultMutableTreeNode
     */
    private DefaultMutableTreeNode getExperiment(final Experiment experiment)
    {
        DefaultMutableTreeNode result = new DefaultMutableTreeNode("Experiment");
        result.add(new DefaultMutableTreeNode("description="
                + experiment.getDescription()));
        result
                .add(new DefaultMutableTreeNode("model="
                        + experiment.getModel()));
        result.add(new DefaultMutableTreeNode("simulator="
                + experiment.getSimulator()));
        result.add(this.getTreatment(experiment.getTreatment()));
        return result;
    }

    /**
     * converts a treatment to a treenode
     * 
     * @param treatment the treatent
     * @return DefaultMutableTreeNode
     */
    private DefaultMutableTreeNode getTreatment(final Treatment treatment)
    {
        DefaultMutableTreeNode result = new DefaultMutableTreeNode("Treatment");
        result.add(new DefaultMutableTreeNode("time unit="
                + treatment.getTimeUnit()));
        Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(treatment.getStartTime());
        String startTime = calendar.getTime().toString();
        result.add(new DefaultMutableTreeNode("start time=" + startTime));
        result.add(this.getProperties(treatment.getProperties()));
        result.add(new DefaultMutableTreeNode("run length="
                + treatment.getRunLength()));
        result.add(new DefaultMutableTreeNode("warmup period="
                + treatment.getWarmupPeriod()));
        for (Replication replication : treatment.getReplications())
        {
            result.add(this.getReplication(replication));
        }
        return result;
    }

    /**
     * converts properties to a treeNode
     * 
     * @param properties the properties
     * @return DefaultMutableTreeNode
     */
    private DefaultMutableTreeNode getProperties(final Properties properties)
    {
        DefaultMutableTreeNode result = new DefaultMutableTreeNode("Properties");
        Enumeration<Object> keys = properties.keys();
        while (keys.hasMoreElements())
        {
            String key = (String) keys.nextElement();
            String value = properties.getProperty(key);
            result.add(new DefaultMutableTreeNode(key + "(" + value + ")"));
        }
        return result;
    }

    /**
     * converts the replication in a tree node
     * 
     * @param replication the replication
     * @return DefaultMutableTreeNode
     */
    private DefaultMutableTreeNode getReplication(final Replication replication)
    {
        DefaultMutableTreeNode result = new DefaultMutableTreeNode(
                "Replication");
        result.add(new DefaultMutableTreeNode("description="
                + replication.getDescription()));
        Map<String, StreamInterface> streams = replication.getStreams();
        for (String key : streams.keySet())
        {
            result.add(new DefaultMutableTreeNode("stream(" + key + ")="
                    + streams.get(key)));
        }
        return result;
    }
}